//+------------------------------------------------------------------+
//|                                                     MACD_Lnx.mq4 |
//|                                                                  |
//|                                                                  |
//|                    Made according Gerald Appel rules by Linuxser |
//| Linuxser 2007                                                    |
//+------------------------------------------------------------------+
#property copyright "Under The GNU General Public License"
#property link      "www.gnu.org"

//---- indicator settings
#property  indicator_separate_window
#property  indicator_buffers 8
#property  indicator_color1  Blue
#property  indicator_color2  Red
#property  indicator_color3  MediumBlue
#property  indicator_color4  Magenta
#property  indicator_color5  Green
#property  indicator_color6  Red
#property  indicator_color7  Magenta
#property  indicator_color8  Lime
#property  indicator_width1  1
#property  indicator_width2  1
#property  indicator_width3  1
#property  indicator_width4  1
#property  indicator_width5  1
#property  indicator_width6  1
#property  indicator_level1  0
#property  indicator_levelcolor  MediumBlue
#property  indicator_levelstyle 0

//---- indicator parameters
extern int FastEMA=13;
extern int SlowEMA=26;
extern int SignalEMA=10;
extern int Applied_Price  =  0; // 0: Close // 1: Open // 2: High // 3: Low // 4: Median (H+L/2) // 5: Typical (H+L+C/3) // 6: Weighted (H+L+C+C/4)
extern int MA_Method  =  1; //MODE_SMA :	0 // MODE_EMA:	1 // MODE_SMMA: 2 // MODE_LWMA: 3 //
extern int Style  = 0; 

//---- indicator buffers
double     MacdBuffer[];
double     SignalBuffer[];
double     StyleBuffer[];
double     OsmaBuffer[];
double     ExtBuffer1[];
double     ExtBuffer2[];
double     ExtBuffer3[];
double     ExtBuffer4[];
double     prev,current;
double     prevosma,currentosma;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- drawing settings
   SetIndexStyle(0,DRAW_LINE);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexStyle(2,DRAW_NONE);
   SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexStyle(4,DRAW_NONE);
   SetIndexStyle(5,DRAW_NONE);
   SetIndexStyle(6,DRAW_NONE);
   SetIndexStyle(7,DRAW_NONE);
   SetIndexDrawBegin(1,SignalEMA);
   IndicatorDigits(Digits);
//---- indicator buffers mapping
   SetIndexBuffer(0,MacdBuffer);
   SetIndexBuffer(1,SignalBuffer);
   SetIndexBuffer(2,StyleBuffer);
   SetIndexBuffer(3,OsmaBuffer);
   SetIndexBuffer(4,ExtBuffer1);
   SetIndexBuffer(5,ExtBuffer2);  
   SetIndexBuffer(6,ExtBuffer3);
   SetIndexBuffer(7,ExtBuffer4);
   
//---- name for DataWindow and indicator subwindow label
   IndicatorShortName("MACD Lnx ("+FastEMA+","+SlowEMA+","+SignalEMA+") Method ["+MA_Method+"])");
   SetIndexLabel(0,"MACD");
   SetIndexLabel(1,"Signal");   
   SetIndexLabel(2,NULL);
   SetIndexLabel(3,"Histogram");
   SetIndexLabel(4,NULL);
   SetIndexLabel(5,NULL);
   SetIndexLabel(6,NULL);

   
   
//---- initialization done
   return(0);
  }
//+------------------------------------------------------------------+
//| Moving Averages Convergence/Divergence                           |
//+------------------------------------------------------------------+
int start()
  {
   int limit;
   int i;
   int counted_bars=IndicatorCounted();
//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;
   
//---- Take care about user mistakes and defaulting on errors  
   if(MA_Method>3) MA_Method=0;
   if(Applied_Price>6) Applied_Price=0;
   if(Style>7) Style=0;   
//---- macd counted in the 1-st buffer
   for(i=0; i<limit; i++)
      MacdBuffer[i]=iMA(NULL,0,FastEMA,0,MA_Method,Applied_Price,i)-iMA(NULL,0,SlowEMA,0,MA_Method,Applied_Price,i);
//---- signal line counted in the 2-nd buffer
   for(i=0; i<limit; i++)
      SignalBuffer[i]=iMAOnArray(MacdBuffer,Bars,SignalEMA,0,MA_Method,i);
//---- main loop
   for(i=0; i<limit; i++)
      OsmaBuffer[i]=MacdBuffer[i]-SignalBuffer[i]; 
//---- dispatch values between 2 buffers
   bool up=true;
   for(i=limit-1; i>=0; i--)
     {
      current=MacdBuffer[i];
      prev=MacdBuffer[i+1];
      if(current>prev) up=true;
      if(current<prev) up=false;
      if(!up)
        {
         ExtBuffer2[i]=current;
         ExtBuffer1[i]=0.0;
        }
      else
        {
         ExtBuffer1[i]=current;
         ExtBuffer2[i]=0.0;
        }
     }
     
   for(i=limit-1; i>=0; i--)
     {
      currentosma=OsmaBuffer[i];
      prevosma=OsmaBuffer[i+1];
      if(currentosma>prevosma) up=true;
      if(currentosma<prevosma) up=false;
      if(!up)
        {
         ExtBuffer3[i]=currentosma;
         ExtBuffer4[i]=0.0;
        }
      else
        {
         ExtBuffer4[i]=currentosma;
         ExtBuffer3[i]=0.0;
        }
     }        
//---- Different Styles      
   if(Style == 1)
      {
         for(i=0; i<limit; i++)
         {
               StyleBuffer[i]=MacdBuffer[i];
               SetIndexStyle(2,DRAW_HISTOGRAM);             
         } 
         }  
    
    if(Style == 2)
      {
         for(i=0; i<limit; i++)
         {
               StyleBuffer[i]=MacdBuffer[i];
               SetIndexStyle(0,DRAW_LINE);
               SetIndexStyle(1,DRAW_NONE);
               SetIndexStyle(2,DRAW_HISTOGRAM);
               SetIndexStyle(3,DRAW_NONE);
               SetIndexLabel(2,"MACD");
               SetIndexLabel(0,NULL);
               SetIndexLabel(1,NULL);
               SetIndexLabel(3,NULL);

         }
         }
    if(Style == 3)
      {
         for(i=0; i<limit; i++)
         {
               SetIndexStyle(0,DRAW_NONE);
               SetIndexStyle(1,DRAW_NONE);
               SetIndexStyle(2,DRAW_NONE);
               SetIndexStyle(3,DRAW_HISTOGRAM);
               SetIndexLabel(3,"MACD");
               SetIndexLabel(0,NULL);
               SetIndexLabel(1,NULL);
               SetIndexLabel(2,NULL);
         }
         }         
    if(Style == 4)
      {
         for(i=0; i<limit; i++)
          {               
               SetIndexStyle(0,DRAW_LINE);
               SetIndexStyle(1,DRAW_LINE);
               SetIndexStyle(2,DRAW_NONE);
               SetIndexStyle(3,DRAW_NONE);
               SetIndexLabel(2,"MACD");
               SetIndexLabel(3,NULL);
         }
         }
    if(Style == 5)
      {
         for(i=0; i<limit; i++)
         {
               
               SetIndexStyle(0,DRAW_NONE);
               SetIndexStyle(1,DRAW_NONE);
               SetIndexStyle(2,DRAW_NONE);
               SetIndexStyle(3,DRAW_NONE);
               SetIndexStyle(4,DRAW_HISTOGRAM);
               SetIndexStyle(5,DRAW_HISTOGRAM);
               SetIndexDrawBegin(4,34);
               SetIndexDrawBegin(5,34);
               SetIndexLabel(4,"MACD Up");
               SetIndexLabel(5,"MACD Down");
               SetIndexLabel(0,NULL);
               SetIndexLabel(1,NULL);
//               SetIndexLabel(2,NULL);
               SetIndexLabel(3,NULL);
         }
         }
     if(Style == 6)
      {
         for(i=0; i<limit; i++)
         {
               SetIndexStyle(0,DRAW_NONE);
               SetIndexStyle(1,DRAW_LINE);
               SetIndexStyle(2,DRAW_NONE);
               SetIndexStyle(3,DRAW_NONE);
               SetIndexStyle(4,DRAW_HISTOGRAM);
               SetIndexStyle(5,DRAW_HISTOGRAM);
               SetIndexDrawBegin(4,34);
               SetIndexDrawBegin(5,34);               
               SetIndexLabel(4,"MACD Up");
               SetIndexLabel(5,"MACD Down"); 
               SetIndexLabel(1,"Signal"); 
               SetIndexLabel(0,NULL);
               SetIndexLabel(2,NULL);
               SetIndexLabel(3,NULL);
 
         }
         }
     
     if(Style == 7)
      {
         for(i=0; i<limit; i++)
         {
               SetIndexStyle(0,DRAW_LINE);
               SetIndexStyle(1,DRAW_LINE);
               SetIndexStyle(2,DRAW_NONE);
               SetIndexStyle(3,DRAW_NONE);
               SetIndexStyle(4,DRAW_NONE);
               SetIndexStyle(5,DRAW_NONE);
               SetIndexStyle(6,DRAW_HISTOGRAM);
               SetIndexStyle(7,DRAW_HISTOGRAM);
               SetIndexDrawBegin(6,34);
               SetIndexDrawBegin(7,34);               
               SetIndexLabel(4,"MACD Up");
               SetIndexLabel(5,"MACD Down"); 
               SetIndexLabel(1,"Signal"); 
               SetIndexLabel(0,NULL);
               SetIndexLabel(2,NULL);
               SetIndexLabel(3,NULL);
 
         }
         }                                  
//---- done
   return(0);
  }
//+------------------------------------------------------------------+